﻿using System.Windows;
using System.Windows.Interactivity;
using System.Collections.Specialized;
using System.Collections;
using System.Collections.ObjectModel;
using Infragistics.Windows.DataPresenter;
using Infragistics.Windows.DataPresenter.Events;

namespace WpfApplication1
{
    public class XamDataGridSelectedItemsBehavior : Behavior<XamDataGrid>
    {
        // Source: The XamDataGrid's SelectedRows collection
        // Target: The object bound as the behaviors SelectedItems collection

        private XamDataGrid Grid
        {
            get
            {
                return AssociatedObject as XamDataGrid;
            }
        }

        public INotifyCollectionChanged SelectedItems
        {
            get { return (INotifyCollectionChanged)GetValue(SelectedItemsProperty); }
            set { SetValue(SelectedItemsProperty, value); }
        }

        public static readonly DependencyProperty SelectedItemsProperty = DependencyProperty.Register(
            "SelectedItems",
            typeof(INotifyCollectionChanged),
            typeof(XamDataGridSelectedItemsBehavior),
            new PropertyMetadata(OnSelectedItemsPropertyChanged));


        protected override void OnAttached()
        {
            base.OnAttached();
            Grid.SelectedItemsChanged += Grid_SelectedItemsChanged;        
        }

        private static void OnSelectedItemsPropertyChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
        {
            var collection = e.NewValue as INotifyCollectionChanged;
            if (collection != null)
            {
                //You should set the SelectedItems of the ComboBox when the SelectedItems property is changed
                XamDataGridSelectedItemsBehavior behavior = obj as XamDataGridSelectedItemsBehavior;

                behavior.UnsubscribeFromEvents();
                XamDataGridSelectedItemsBehavior.TransferTargetToSource(collection as IList, behavior.Grid);
                behavior.SubscribeToEvents();
            }
        }

        void SelectedItems_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            UnsubscribeFromEvents();
            TransferTargetToSource(this.SelectedItems as IList, Grid);
            SubscribeToEvents();
        }

        void Grid_SelectedItemsChanged(object sender, Infragistics.Windows.DataPresenter.Events.SelectedItemsChangedEventArgs e)
        {
            UnsubscribeFromEvents();
            TransferSourceToTarget(Grid.SelectedItems.Records, this.SelectedItems as IList);
            SubscribeToEvents();
        }

        private void SubscribeToEvents()
        {
            Grid.SelectedItemsChanged += new System.EventHandler<Infragistics.Windows.DataPresenter.Events.SelectedItemsChangedEventArgs>(Grid_SelectedItemsChanged);
            if (SelectedItems != null)
                SelectedItems.CollectionChanged += SelectedItems_CollectionChanged;
        }

        private void UnsubscribeFromEvents()
        {
            Grid.SelectedItemsChanged -= new System.EventHandler<Infragistics.Windows.DataPresenter.Events.SelectedItemsChangedEventArgs>(Grid_SelectedItemsChanged);
            if (SelectedItems != null)
                SelectedItems.CollectionChanged -= SelectedItems_CollectionChanged;
        }

        public static void TransferSourceToTarget(SelectedItemCollectionBase source, IList target)
        {
            if (source == null || target == null)
                return;

            target.Clear();

            foreach (Record r in source)
            {
                // make sure we only take data rows from the root band 
                if (r.IsDataRecord)
                {
                    target.Add((r as DataRecord).DataItem);
                }
            }
        }

        public static void TransferTargetToSource(IList target, XamDataGrid source)
        {
            if (source == null || target == null)
                return;

            source.SelectedItems.Records.Clear();

            foreach (object obj in target)
            {
                foreach (Record record in source.Records)
                {
                    if ((record as DataRecord).DataItem == obj)
                        source.SelectedItems.Records.Add(record);
                }
            }

            // If selected items is still empty then we don't have an active record anymore.
            if (source.SelectedItems.Records.Count == 0)
                source.ActiveRecord = null;
        }
    }
}